/*#############################################################
File Name       : ex1_analog_leds.c
Author          : Grant Phillips
Date Modified   : 08/09/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F0-Discovery

Description     : Example that measures an analog voltage on 
                  PA1 (ADC_IN1) and displays the result on
                  8 LEDs.      

Requirements    : * STM32F0-Discovery Board
									* NB! NB! NB! NB! NB! NB! NB! NB! NB! NB! NB!
										Open the "stm32f0xx.h" file in the project
										folder and remove the comments around the
										#include "stm32f0xx_adc.h" entry on line 34.
										This will make sure that the ADC peripheral
										library will be included into all your future
										programs.
              
Circuit         : * 8 LEDs connected to PC7 - PC0
									* a potentiometer (POT) connected to PA1 
                    (ADC_IN1 - Analog Channel 1) 

See the STM32F0-Discovery User Manual (UM1525) for the block 
diagram of the STM32F051R8T6 processor (p13), a summary of
the GPIO connections (p20-26) and the schematic diagram (p37-39)

##############################################################*/

/* #includes -------------------------------------------------*/
#include "main.h"																								//main library to include for device drivers, peripheral drivers, etc.

/* #defines --------------------------------------------------*/

/* #function prototypes --------------------------------------*/

/* #global variables -----------------------------------------*/



// Unused global variables that have to be included to ensure correct compiling */
// ###### DO NOT CHANGE ######
// ===============================================================================
__IO uint32_t TimingDelay = 0;																	//used with the Delay function
__IO uint8_t DataReady = 0;
__IO uint32_t USBConnectTimeOut = 100;
__IO uint32_t UserButtonPressed = 0;
__IO uint8_t PrevXferComplete = 1;
// ===============================================================================



int main(void)
{
	GPIO_InitTypeDef        GPIO_InitStructure;										//structure used for setting up a GPIO port
	
	ADC_InitTypeDef       	ADC_InitStructure;										//structure used for setting up the ADC
	
	unsigned int ADC_val;																					//variable for holding the 12-bit ADC result
	__IO unsigned int ADC_calibration_value;											//variable for storing the calibration value
																																//the __IO just indicates to the compiler that the
																																//variable is volatile and is used by other processes not in
																																//program - otherwise it ignores it when compiling, because
																																//this vriable is not really used in this program

	/* GPIOC Periph clock enable */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);						//enable the AHB bus to use GPIOC
  
  /* Configure PC7 - PC0 in output pushpull mode*/
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7 | GPIO_Pin_6 | GPIO_Pin_5 | GPIO_Pin_4 | GPIO_Pin_3 | GPIO_Pin_2 | GPIO_Pin_1 | GPIO_Pin_0;	//which pins to setup, seperated by |
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;									//setup for output mode
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;								//push-pull mode; also available is GPIO_OType_OD (open drain)
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;							//update speed is 50Mhz; also available is GPIO_Speed_10MHz and GPIO_Speed_2MHz
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;							//output not pulled up or down
  GPIO_Init(GPIOC, &GPIO_InitStructure);												//initialize GPIOC with the above structure
	
	
/* ADC1 Periph clock enable */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);
						
	/* ADC Channel configuration */
  /* GPIOA Periph clock enable */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

  /* Configure ADC1 Channel5 (PA1) as analog input */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 ;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  
	/* Fill each ADC_InitStructure member with its default value */
  ADC_StructInit(&ADC_InitStructure);

	/* Configure the ADC1 in continous mode withe a resolutuion equal to 12 bits*/
  ADC_InitStructure.ADC_Resolution = ADC_Resolution_12b;													//12-bit resolution
  ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;															//enable continous mode
  ADC_InitStructure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None;    	//ADC No external trigger for regular conversion
  ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_T1_TRGO;					//ADC external trigger event 0
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;													//ADC Data alignment right 
  ADC_InitStructure.ADC_ScanDirection = ADC_ScanDirection_Upward;									//scan ADC channels upward
  ADC_Init(ADC1, &ADC_InitStructure);
	
	/* Convert the ADC1 Channel 1 with 7.5 Cycles as sampling time */ 
  ADC_ChannelConfig(ADC1, ADC_Channel_1 , ADC_SampleTime_7_5Cycles);

	 /* ADC Calibration */
  ADC_GetCalibrationFactor(ADC1);

  /* Enable the auto delay feature */    
  ADC_WaitModeCmd(ADC1, ENABLE); 
  
  /* Enable the Auto power off mode */
  ADC_AutoPowerOffCmd(ADC1, ENABLE); 
  
  /* Enable ADCperipheral[PerIdx] */
  ADC_Cmd(ADC1, ENABLE);     
  
  /* Wait the ADEN flag */
  while(!ADC_GetFlagStatus(ADC1, ADC_FLAG_ADEN)); 
   
  /* ADC1 regular Software Start Conv */ 
  ADC_StartOfConversion(ADC1); 	
	
	/* Main program loop */
  while (1)
  {
		/* Test EOC (EndOfConversion flag. Wait for the ADC to finish converting a analog input */
    while(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);
    
    /* Get ADC1 converted data */
    ADC_val = ADC_GetConversionValue(ADC1);
		
		/* Write the 12-bit answer to the LEDs.  But in this example we only have 8 LEDs, so 
		for demonstration purposes we will shift all the bits of the result 4 spaces to right,
		so that only the higher 8 bits of the 12-bit result is displayed on the LEDs */
		GPIO_Write(GPIOC, ADC_val>>4);
	}	
}

// -------------------------------------------------------------------------------

// Function to insert a timing delay of nTime
// ###### DO NOT CHANGE ######
void Delay(__IO uint32_t nTime)
{
  TimingDelay = nTime;

  while(TimingDelay != 0);
}

// Function to Decrement the TimingDelay variable.
// ###### DO NOT CHANGE ######
void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}

// Unused functions that have to be included to ensure correct compiling
// ###### DO NOT CHANGE ######
// =======================================================================
uint32_t L3GD20_TIMEOUT_UserCallback(void)
{
  return 0;
}

uint32_t LSM303DLHC_TIMEOUT_UserCallback(void)
{
  return 0;
}
// =======================================================================



