/*#############################################################
File Name       : ex5_analog_6channels_LM6029ACW.c
Author          : Grant Phillips
Date Modified   : 08/09/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F0-Discovery

Description     : Example that measures 6 analog voltages, all
									6 on ADC1.  DMA requests are required 
									to efectively do simultaneous conversions of 
									multiple channels.      

Requirements    : * STM32F0-Discovery Board
									* Make sure the LM6029ACW_F0.c file is in the 
                    same folder than the project
									* Define the SET and RESET functions for the
										output port pins in the LM6029ACW_F0.c
										library
									* Do the proper GPIO initialization for the
										output port pins in LM6029ACW_Init() in the
										LM6029ACW_F0.c library
              
Circuit         : * potentiometers (POTs) connected to: 
										- PA1 (ADC1_IN1 - Analog1 Channel 1) 
										- PA2 (ADC1_IN2 - Analog1 Channel 2)
										- PA3 (ADC1_IN3 - Analog1 Channel 3)
										- PA4 (ADC1_IN4 - Analog1 Channel 4)
										- PA5 (ADC1_IN5 - Analog1 Channel 5)
										- PA6 (ADC1_IN6 - Analog1 Channel 6) 
									* The LM6029ACW is connected as described in
										ex1_LM6029ACW_HelloWorld.c

See the STM32F0-Discovery User Manual (UM1525) for the block 
diagram of the STM32F051R8T6 processor (p13), a summary of
the GPIO connections (p20-26) and the schematic diagram (p37-39)

##############################################################*/

/* #includes -------------------------------------------------*/
#include "main.h"																								//main library to include for device drivers, peripheral drivers, etc.
#include "LM6029ACW_F0.c"																				//for the LM6029ACW graphics lcd functions on the STM32F0-Discovery

/* #defines --------------------------------------------------*/

/* #function prototypes --------------------------------------*/
void ADC1_6CH_DMA_Config(void);																	//prototype for the user function to initialize ADC1 in DMA mode

/* #global variables -----------------------------------------*/

__IO uint16_t ADC1_val[6];																			//array to hold the 2 converted values 
																																//ADC1_val[0] - PA6, ADC1_val[1] - PA1, ADC1_val[2] - PA2, ADC1_val[3] - PA3, ADC1_val[4] - PA4, ADC1_val[5] - PA5,

// Unused global variables that have to be included to ensure correct compiling */
// ###### DO NOT CHANGE ######
// ===============================================================================
__IO uint32_t TimingDelay = 0;																	//used with the Delay function
__IO uint8_t DataReady = 0;
__IO uint32_t USBConnectTimeOut = 100;
__IO uint32_t UserButtonPressed = 0;
__IO uint8_t PrevXferComplete = 1;
// ===============================================================================



int main(void)
{
	char buf[21];
	
	RCC_ClocksTypeDef 			RCC_Clocks;														//structure used for setting up the SysTick Interrupt
	
	/* Set the SysTick Interrupt to occur every 1ms) */
	RCC_GetClocksFreq(&RCC_Clocks);
	if (SysTick_Config(RCC_Clocks.HCLK_Frequency / 1000))
		while(1);
	
	LM6029ACW_Init();																							//initialize the lcd
	ADC1_6CH_DMA_Config();																				//initialize ADC1 in DMA mode
	
	/* Main program loop */
  while (1)
  {
		/* due to the circular buffer mode of the DMA, the first channel (ADC_IN1) will
		be stored in ADC1_val[1], ADC_IN2 in ADC1_val[2], ADC_IN3 in ADC1_val[3],
		ADC_IN4 in ADC1_val[4], ADC_IN5 in ADC1_val[5] and ADC_IN6 in ADC1_val[0] */
		
		sprintf(buf, "ADC1_IN1 : %4d", ADC1_val[1]);
		LM6029ACW_GotoXY(0,0);
		LM6029ACW_PutStr(buf, 1);
		
		sprintf(buf, "ADC1_IN2 : %4d", ADC1_val[2]);
		LM6029ACW_GotoXY(0,1);
		LM6029ACW_PutStr(buf, 1);
		
		sprintf(buf, "ADC1_IN3 : %4d", ADC1_val[3]);
		LM6029ACW_GotoXY(0,2);
		LM6029ACW_PutStr(buf, 1);
		
		sprintf(buf, "ADC1_IN4 : %4d", ADC1_val[4]);
		LM6029ACW_GotoXY(0,3);
		LM6029ACW_PutStr(buf, 1);
		
		sprintf(buf, "ADC1_IN5 : %4d", ADC1_val[5]);
		LM6029ACW_GotoXY(0,4);
		LM6029ACW_PutStr(buf, 1);
		
		sprintf(buf, "ADC1_IN6 : %4d", ADC1_val[0]);
		LM6029ACW_GotoXY(0,5);
		LM6029ACW_PutStr(buf, 1);
		
		Delay(10);
	}	
}


/* User define function to setup the ADC1 for this application */
void ADC1_6CH_DMA_Config(void)
{
  ADC_InitTypeDef     ADC_InitStructure;
  DMA_InitTypeDef     DMA_InitStructure;															//structure for setting up the DMA
  GPIO_InitTypeDef  GPIO_InitStructure;
	
	/* Enable ADC1, DMA1 and GPIOC clock */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
	
	/* Configure ADC1 Channel1 (PA1) and Channel2 (PA2) as analog input */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_2 |
																GPIO_Pin_3 | GPIO_Pin_4 |
																GPIO_Pin_5 | GPIO_Pin_6;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
	
  /* ADC1 DeInit */  
  ADC_DeInit(ADC1);
  
  /* ADC1 Periph clock enable */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);
  
  /* DMA1 clock enable */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1 , ENABLE);
  
  /* DMA1 Channel1 Config */
  DMA_DeInit(DMA1_Channel1);
  DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)&ADC1->DR;								//specify the base address of the peripheral (ADC1) to connect to the DMA1 channel 1
  DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)ADC1_val;										//specify the base address of the result array
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
  DMA_InitStructure.DMA_BufferSize = 6;																					//=6 because we are using 6 channels and each channel is stored in an array element
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;												//automatically increase the element number for each channel conversion
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;		//16bit size
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;						//16bit size
  DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;																//after all the channels are read by the DMA, start again
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;
  DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
  DMA_Init(DMA1_Channel1, &DMA_InitStructure);
  
  /* DMA1 Channel1 enable */
  DMA_Cmd(DMA1_Channel1, ENABLE);
  
  /* ADC DMA request in circular mode */
  ADC_DMARequestModeConfig(ADC1, ADC_DMAMode_Circular);
  
  /* Enable ADC_DMA */
  ADC_DMACmd(ADC1, ENABLE);  
  
  /* Initialize ADC structure */
  ADC_StructInit(&ADC_InitStructure);
  
  /* Configure the ADC1 in continous mode withe a resolutuion equal to 12 bits  */
  ADC_InitStructure.ADC_Resolution = ADC_Resolution_12b;
  ADC_InitStructure.ADC_ContinuousConvMode = ENABLE; 
  ADC_InitStructure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None;
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
  ADC_InitStructure.ADC_ScanDirection = ADC_ScanDirection_Upward;								//scan upward
  ADC_Init(ADC1, &ADC_InitStructure); 
  
	/* Convert the ADC1 Channel 1 to 6 with 7.5 Cycles as sampling time */ 
  ADC_ChannelConfig(ADC1, ADC_Channel_1 , ADC_SampleTime_7_5Cycles);
	ADC_ChannelConfig(ADC1, ADC_Channel_2 , ADC_SampleTime_7_5Cycles);
	ADC_ChannelConfig(ADC1, ADC_Channel_3 , ADC_SampleTime_7_5Cycles);
	ADC_ChannelConfig(ADC1, ADC_Channel_4 , ADC_SampleTime_7_5Cycles);
	ADC_ChannelConfig(ADC1, ADC_Channel_5 , ADC_SampleTime_7_5Cycles);
	ADC_ChannelConfig(ADC1, ADC_Channel_6 , ADC_SampleTime_7_5Cycles);
	
  /* ADC Calibration */
  ADC_GetCalibrationFactor(ADC1);
  
  /* Enable ADC1 */
  ADC_Cmd(ADC1, ENABLE);     
  
  /* Wait the ADCEN flag */
  while(!ADC_GetFlagStatus(ADC1, ADC_FLAG_ADEN)); 
  
  /* ADC1 regular Software Start Conv */ 
  ADC_StartOfConversion(ADC1);
}









// -------------------------------------------------------------------------------

// Function to insert a timing delay of nTime
// ###### DO NOT CHANGE ######
void Delay(__IO uint32_t nTime)
{
  TimingDelay = nTime;

  while(TimingDelay != 0);
}

// Function to Decrement the TimingDelay variable.
// ###### DO NOT CHANGE ######
void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}

// Unused functions that have to be included to ensure correct compiling
// ###### DO NOT CHANGE ######
// =======================================================================
uint32_t L3GD20_TIMEOUT_UserCallback(void)
{
  return 0;
}

uint32_t LSM303DLHC_TIMEOUT_UserCallback(void)
{
  return 0;
}
// =======================================================================



